//*****************************************************************************
//                                 DlgGenCfg.cpp                              *
//                                ---------------                             *
//  Started     : 27/02/2004                                                  *
//  Last Update : 24/08/2006                                                  *
//  Copyright   : (C) 2004 by M.S.Waters                                      *
//  Email       : M.Waters@bom.gov.au                                         *
//*****************************************************************************

//*****************************************************************************
//                                                                            *
//    This program is free software; you can redistribute it and/or modify    *
//    it under the terms of the GNU General Public License as published by    *
//    the Free Software Foundation; either version 2 of the License, or       *
//    (at your option) any later version.                                     *
//                                                                            *
//*****************************************************************************

#include "gnucap/DlgGenCfg.hpp"

//*****************************************************************************
// Implement an event table.

BEGIN_EVENT_TABLE( DlgGenCfg, wxDialog )

  EVT_BUTTON( ID_BTN_OK,     DlgGenCfg::OnBtnOk     )
  EVT_BUTTON( ID_BTN_CANCEL, DlgGenCfg::OnBtnCancel )

END_EVENT_TABLE( )

//*****************************************************************************
// Constructor.
//
// Argument List:
//   poWin - A pointer to the dialog parent window

DlgGenCfg::DlgGenCfg( wxWindow * poWin ) : wxDialog( poWin, -1, wxT(""),
    wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE, wxDialogNameStr )
{
  SetTitle( wxT("Generator Setup") );
  SetSize( wxSize( 290, 438 ) );
  Initialize( );
  bClear( );
  bSetDefaults( );
}

//*****************************************************************************
// Destructor.

DlgGenCfg::~DlgGenCfg( )
{
}

//*****************************************************************************
// Initialize object attributes.

void  DlgGenCfg::Initialize( void )
{
  wxPanel * poPnl;

  // Create a panel to contain the total parameter controls
  poPnl = new wxPanel( this );

  m_oPnlAmp   .bCreate( poPnl, ID_PNL_AMP,    95 );
  m_oPnlOffset.bCreate( poPnl, ID_PNL_OFFSET, 95 );
 
  m_oPnlAmp   .bSetName( wxT("Amplitude") );
  m_oPnlOffset.bSetName( wxT("DC Offset") );

  m_oPnlAmp   .bSetUType( ChoUnits::eUNTS_SCLR );
  m_oPnlOffset.bSetUType( ChoUnits::eUNTS_VOLT );

  // Create a panel to contain the sine wave parameter controls
  poPnl = new wxPanel( this );

  m_oPnlFreq  .bCreate( poPnl, ID_PNL_FREQ,  95 );
  m_oPnlPhase .bCreate( poPnl, ID_PNL_PHASE, 95 );

  m_oPnlFreq  .bSetName( wxT("Frequency")     );
  m_oPnlPhase .bSetName( wxT("Initial Phase") );

  m_oPnlFreq  .bSetUType( ChoUnits::eUNTS_FREQ  );
  m_oPnlPhase .bSetUType( ChoUnits::eUNTS_PHASE );

  // Create a panel to contain the pulse parameter controls
  poPnl = new wxPanel( this );

  m_oPnlInitV .bCreate( poPnl, ID_PNL_INITV,  95 );
  m_oPnlMinV  .bCreate( poPnl, ID_PNL_MINV,   95 );
  m_oPnlMaxV  .bCreate( poPnl, ID_PNL_MAXV,   95 );
  m_oPnlDelay .bCreate( poPnl, ID_PNL_DELAY,  95 );
  m_oPnlRise  .bCreate( poPnl, ID_PNL_RISE,   95 );
  m_oPnlWidth .bCreate( poPnl, ID_PNL_WIDTH,  95 );
  m_oPnlFall  .bCreate( poPnl, ID_PNL_FALL,   95 );
  m_oPnlPeriod.bCreate( poPnl, ID_PNL_PERIOD, 95 );

  m_oPnlInitV .bSetName( wxT("Initial Value") );
  m_oPnlMinV  .bSetName( wxT("Minimum Level") );
  m_oPnlMaxV  .bSetName( wxT("Maximum Level") );
  m_oPnlDelay .bSetName( wxT("Initial Delay") );
  m_oPnlRise  .bSetName( wxT("Rise Time")     );
  m_oPnlWidth .bSetName( wxT("Pulse Width")   );
  m_oPnlFall  .bSetName( wxT("Fall Time")     );
  m_oPnlPeriod.bSetName( wxT("Period")        );

  m_oPnlInitV .bSetUType( ChoUnits::eUNTS_VOLT );
  m_oPnlMinV  .bSetUType( ChoUnits::eUNTS_VOLT );
  m_oPnlMaxV  .bSetUType( ChoUnits::eUNTS_VOLT );
  m_oPnlDelay .bSetUType( ChoUnits::eUNTS_TIME );
  m_oPnlRise  .bSetUType( ChoUnits::eUNTS_TIME );
  m_oPnlWidth .bSetUType( ChoUnits::eUNTS_TIME );
  m_oPnlFall  .bSetUType( ChoUnits::eUNTS_TIME );
  m_oPnlPeriod.bSetUType( ChoUnits::eUNTS_TIME );

  m_oPnlFreq.bSetRange( 0.0, 1000.0 );
  m_oPnlFreq.bSetUnits( wxT("Hz") );

  // Create a panel for the buttons
  poPnl = new wxPanel( this );
  m_oBtnOk    .Create( poPnl, ID_BTN_OK,     wxT("OK")     );
  m_oBtnCancel.Create( poPnl, ID_BTN_CANCEL, wxT("Cancel") );

  // Layout the of the display objects
  DoLayout( );

  // Initialize temporary storage with the spin control values
  GetValues( );
}

//*****************************************************************************
// Layout the dialog display objects.

void  DlgGenCfg::DoLayout( void )
{
  wxPanel * poPnl;
  wxBoxSizer * poSzr;
  wxSizerFlags  oFlags;

  // Create and set the underlying dialogue sizer
  poSzr = new wxBoxSizer( wxVERTICAL );
  SetSizer( poSzr );

  // Create and set the sizer for the pulse parameter controls
  poPnl = (wxPanel *) m_oPnlAmp.GetParent( );
  poSzr = new wxStaticBoxSizer( wxVERTICAL, poPnl, wxT("Total") );

  oFlags.Center( );
  oFlags.Border( wxTOP | wxLEFT | wxRIGHT, 10 );
  poSzr->Add( &m_oPnlAmp,    oFlags );
  oFlags.Border( wxBOTTOM, 5 );
  poSzr->Add( &m_oPnlOffset, oFlags );

  poPnl->SetSizer( poSzr );
  poSzr->SetSizeHints( poPnl );
  oFlags.Border( wxTOP | wxLEFT | wxRIGHT, 10 );
  GetSizer( )->Add( poPnl, oFlags );

  // Create and set the sizer for the sine wave parameter controls
  poPnl = (wxPanel *) m_oPnlFreq.GetParent( );
  poSzr = new wxStaticBoxSizer( wxVERTICAL, poPnl, wxT("Sine Wave") );

  oFlags.Border( wxTOP | wxLEFT | wxRIGHT, 10 );
  poSzr->Add( &m_oPnlFreq,  oFlags );
  oFlags.Border( wxBOTTOM, 5 );
  poSzr->Add( &m_oPnlPhase, oFlags );

  poPnl->SetSizer( poSzr );
  poSzr->SetSizeHints( poPnl );
  oFlags.Border( wxTOP | wxLEFT | wxRIGHT, 10 );
  GetSizer( )->Add( poPnl, oFlags );

  // Create and set the sizer for the pulse parameter controls
  poPnl = (wxPanel *) m_oPnlInitV.GetParent( );
  poSzr = new wxStaticBoxSizer( wxVERTICAL, poPnl, wxT("Pulse") );

  oFlags.Border( wxTOP, 10 );
  poSzr->Add( &m_oPnlInitV,  oFlags );
  oFlags.Border( wxLEFT | wxRIGHT, 10 );
  poSzr->Add( &m_oPnlMinV,   oFlags );
  poSzr->Add( &m_oPnlMaxV,   oFlags );
  poSzr->Add( &m_oPnlDelay,  oFlags );
  poSzr->Add( &m_oPnlRise,   oFlags );
  poSzr->Add( &m_oPnlWidth,  oFlags );
  poSzr->Add( &m_oPnlFall,   oFlags );
  oFlags.Border( wxBOTTOM, 5 );
  poSzr->Add( &m_oPnlPeriod, oFlags );

  poPnl->SetSizer( poSzr );
  poSzr->SetSizeHints( poPnl );
  oFlags.Border( wxALL, 10 );
  GetSizer( )->Add( poPnl, oFlags );

  // Create and set the sizer for the buttons
  poPnl = (wxPanel *) m_oBtnOk.GetParent( );
  poSzr = new wxBoxSizer( wxHORIZONTAL );

  oFlags.Border( wxTOP | wxLEFT | wxRIGHT, 10 );
  poSzr->Add( &m_oBtnOk,     oFlags );
  poSzr->AddSpacer( 10 );
  poSzr->Add( &m_oBtnCancel, oFlags );

  poPnl->SetSizer( poSzr );
  poSzr->SetSizeHints( poPnl );
  oFlags.Border( wxBOTTOM, 20 );
  GetSizer( )->Add( poPnl, oFlags );

  // Set dialogues minimum size and initial size as calculated by the sizer
  GetSizer( )->SetSizeHints( this );
}

//*****************************************************************************
// Get a reference to a value panel associated with a control ID number.
//
// Argument List:
//   iPnlID - The control ID number
//
// Return Values:
//   A reference to a value panel

PnlValue & DlgGenCfg::roGetPanel( int iPnlID )
{
  // No argument tests here so use this function responsibly or suffer the pain
  switch( iPnlID )
  {
    case ID_PNL_AMP    : return( m_oPnlAmp    );
    case ID_PNL_OFFSET : return( m_oPnlOffset );
    case ID_PNL_FREQ   : return( m_oPnlFreq   );
    case ID_PNL_PHASE  : return( m_oPnlPhase  );
    case ID_PNL_INITV  : return( m_oPnlInitV  );
    case ID_PNL_MINV   : return( m_oPnlMinV   );
    case ID_PNL_MAXV   : return( m_oPnlMaxV   );
    case ID_PNL_DELAY  : return( m_oPnlDelay  );
    case ID_PNL_RISE   : return( m_oPnlRise   );
    case ID_PNL_WIDTH  : return( m_oPnlWidth  );
    case ID_PNL_FALL   : return( m_oPnlFall   );
    case ID_PNL_PERIOD : return( m_oPnlPeriod );
    default            : return( m_oPnlFreq   );
  }
}

//*****************************************************************************
// Set the values in the spin controls from temporary storage.

void  DlgGenCfg::SetValues( void )
{
  int  i1;

  SetEvtHandlerEnabled( FALSE );

  for( i1=ID_PNL_FST; i1<=ID_PNL_LST; i1++ )
    roGetPanel( i1 ).bSetValue( (float) m_dfPnlValue[ i1 ] );

  SetEvtHandlerEnabled( TRUE );
}

//*****************************************************************************
// Get the values from the spin controls and put them in temporary storage.

void  DlgGenCfg::GetValues( void )
{
  int  i1;

  SetEvtHandlerEnabled( FALSE );

  for( i1=ID_PNL_FST; i1<=ID_PNL_LST; i1++ )
    m_dfPnlValue[ i1 ] = roGetPanel( i1 ).dfGetValue( );

  SetEvtHandlerEnabled( TRUE );
}

//*****************************************************************************
// Test the values against each other.
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  DlgGenCfg::bValidate( void )
{
  wxString  osErrMsg;
  double    df1, df2, df3, df4;

  // Test the pulse time values
  df1 = m_oPnlRise  .dfGetValue( );
  df2 = m_oPnlWidth .dfGetValue( );
  df3 = m_oPnlFall  .dfGetValue( );
  df4 = m_oPnlPeriod.dfGetValue( );
  if(      df4 < df2 )
    osErrMsg << wxT("The rise time is greater than the pulse period.\n");
  else if( df4 < df3 )
    osErrMsg << wxT("The pulse width is greater than the pulse period.\n");
  else if( df4 < df4 )
    osErrMsg << wxT("The fall time is greater than the pulse period.\n");
  else if( df4 < df1+df2+df3 )
    osErrMsg << wxT("The pulse period is less then the total pulse width.\n");

  // Test the pulse level values
  df1 = m_oPnlInitV.dfGetValue( );
  df2 = m_oPnlMinV .dfGetValue( );
  df3 = m_oPnlMaxV .dfGetValue( );
  if(      df1 < df2 )
    osErrMsg << wxT("The initial value is less than the minimum value.\n");
  else if( df1 > df3 )
    osErrMsg << wxT("The initial value is greater than the maximum value.\n");
  else if( df2 > df3 )
    osErrMsg << wxT("The minimum value is less than the maximum value.\n");

  // Test overall logic
  if( df4>0.0  && (df2==0.0 && df3==0.0) )
    osErrMsg << wxT("Pulse timing has been defined but no levels.\n");
  if( df4==0.0 && (df2!=0.0 || df3!=0.0) )
    osErrMsg << wxT("Pulse levels have been defined but no timing.\n");

  if( ! osErrMsg.IsEmpty( ) )
  {
    wxMessageDialog  oMsgDlg( this, osErrMsg, wxT("Error"), wxOK | wxICON_ERROR );
    oMsgDlg.ShowModal( );
  }

  return( osErrMsg.IsEmpty( ) );
}

//*****************************************************************************
// Parse a generator command string.
//
// Argument List:
//   rosCmd - The generator command to be parsed
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  DlgGenCfg::bSetGenCmd( const wxString & rosCmd )
{
  wxStringTokenizer  ostk1;
  wxString  os1, os2;

  // Extract the generator command parameter list
  ostk1.SetString( rosCmd, wxT(" =\t\r\n") );
  if( ostk1.CountTokens( ) < 3 )        return( FALSE );
  os1 = ostk1.GetNextToken( ).Upper( );
  if( ! os1.StartsWith( wxT(".GEN") ) ) return( FALSE );

  // Extract each generator parameter value
  while( ostk1.HasMoreTokens( ) )
  {
    os1 = ostk1.GetNextToken( ).Upper( );
    if( ! ostk1.HasMoreTokens( ) ) break;
    os2 = ostk1.GetNextToken( );
    if(      os1.StartsWith( wxT("AM") ) ) m_oPnlAmp   .bSetValue( os2 );
    else if( os1.StartsWith( wxT("OF") ) ) m_oPnlOffset.bSetValue( os2 );
    else if( os1.StartsWith( wxT("FR") ) ) m_oPnlFreq  .bSetValue( os2 );
    else if( os1.StartsWith( wxT("PH") ) ) m_oPnlPhase .bSetValue( os2 );
    else if( os1.StartsWith( wxT("IN") ) ) m_oPnlInitV .bSetValue( os2 );
    else if( os1.StartsWith( wxT("MI") ) ) m_oPnlMinV  .bSetValue( os2 );
    else if( os1.StartsWith( wxT("MA") ) ) m_oPnlMaxV  .bSetValue( os2 );
    else if( os1.StartsWith( wxT("DE") ) ) m_oPnlDelay .bSetValue( os2 );
    else if( os1.StartsWith( wxT("RI") ) ) m_oPnlRise  .bSetValue( os2 );
    else if( os1.StartsWith( wxT("WI") ) ) m_oPnlWidth .bSetValue( os2 );
    else if( os1.StartsWith( wxT("FA") ) ) m_oPnlFall  .bSetValue( os2 );
    else if( os1.StartsWith( wxT("PE") ) ) m_oPnlPeriod.bSetValue( os2 );
  }

  return( TRUE );
}

//*****************************************************************************
// Create the generator command string.
//
// Return Values:
//   The generator command

const wxString & DlgGenCfg::rosGetGenCmd( void )
{
  m_osGenCmd = wxT(".GENERATOR");

  if( roGetPanel( ID_PNL_AMP ).IsEnabled( ) )
    m_osGenCmd << wxT(" AMP=")    << roGetPanel( ID_PNL_AMP    ).rosGetValue( );
  if( roGetPanel( ID_PNL_OFFSET ).IsEnabled( ) )
    m_osGenCmd << wxT(" OFFSET=") << roGetPanel( ID_PNL_OFFSET ).rosGetValue( );

  if( roGetPanel( ID_PNL_FREQ ).dfGetValue( ) != 0.0 )
  {
    if( roGetPanel( ID_PNL_FREQ ).IsEnabled( ) )
      m_osGenCmd << wxT(" FREQ=")  << roGetPanel( ID_PNL_FREQ  ).rosGetValue( );
    if( roGetPanel( ID_PNL_PHASE ).IsEnabled( ) )
      m_osGenCmd << wxT(" PHASE=") << roGetPanel( ID_PNL_PHASE ).rosGetValue( );
  }

  if( roGetPanel( ID_PNL_MAXV ).dfGetValue( ) != 0.0 )
  {
    if( roGetPanel( ID_PNL_INITV ).IsEnabled( ) )
      m_osGenCmd << wxT(" INIT=")   << roGetPanel( ID_PNL_INITV  ).rosGetValue( );
    if( roGetPanel( ID_PNL_MINV ).IsEnabled( ) )
      m_osGenCmd << wxT(" MIN=")    << roGetPanel( ID_PNL_MINV   ).rosGetValue( );
    if( roGetPanel( ID_PNL_MAXV ).IsEnabled( ) )
      m_osGenCmd << wxT(" MAX=")    << roGetPanel( ID_PNL_MAXV   ).rosGetValue( );
    if( roGetPanel( ID_PNL_DELAY ).IsEnabled( ) )
      m_osGenCmd << wxT(" DELAY=")  << roGetPanel( ID_PNL_DELAY  ).rosGetValue( );
    if( roGetPanel( ID_PNL_RISE ).IsEnabled( ) )
      m_osGenCmd << wxT(" RISE=")   << roGetPanel( ID_PNL_RISE   ).rosGetValue( );
    if( roGetPanel( ID_PNL_WIDTH ).IsEnabled( ) )
      m_osGenCmd << wxT(" WIDTH=")  << roGetPanel( ID_PNL_WIDTH  ).rosGetValue( );
    if( roGetPanel( ID_PNL_FALL ).IsEnabled( ) )
      m_osGenCmd << wxT(" FALL=")   << roGetPanel( ID_PNL_FALL   ).rosGetValue( );
    if( roGetPanel( ID_PNL_PERIOD ).IsEnabled( ) )
      m_osGenCmd << wxT(" PERIOD=") << roGetPanel( ID_PNL_PERIOD ).rosGetValue( );
  }

  return( m_osGenCmd );
}

//*****************************************************************************
// Reset all dialog settings to defaults.
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  DlgGenCfg::bClear( void )
{
  bool  bRtn=TRUE;

  // Set default spin control values
  if( ! m_oPnlAmp   .bClear( ) ) bRtn = FALSE;
  if( ! m_oPnlOffset.bClear( ) ) bRtn = FALSE;
  if( ! m_oPnlFreq  .bClear( ) ) bRtn = FALSE;
  if( ! m_oPnlPhase .bClear( ) ) bRtn = FALSE;
  if( ! m_oPnlInitV .bClear( ) ) bRtn = FALSE;
  if( ! m_oPnlMinV  .bClear( ) ) bRtn = FALSE;
  if( ! m_oPnlMaxV  .bClear( ) ) bRtn = FALSE;
  if( ! m_oPnlDelay .bClear( ) ) bRtn = FALSE;
  if( ! m_oPnlRise  .bClear( ) ) bRtn = FALSE;
  if( ! m_oPnlWidth .bClear( ) ) bRtn = FALSE;
  if( ! m_oPnlFall  .bClear( ) ) bRtn = FALSE;
  if( ! m_oPnlPeriod.bClear( ) ) bRtn = FALSE;

  return( bRtn );
}

//*****************************************************************************
// Enable or disable a value panel.
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  DlgGenCfg::bEnablePanel( int iPnlID, bool bState )
{
  if( iPnlID<ID_PNL_FST || iPnlID>ID_PNL_LST ) return( FALSE );

  if( roGetPanel( iPnlID ).Enable( bState ) )  return( FALSE );

  return( TRUE );
}

//*****************************************************************************
// Set the value on a panel.
//
// Argument List:
//   iPnlID   - Spin control identifier
//   fValue   - The value to set the spin control
//   rosUnits - The units to display
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  DlgGenCfg::bSetValue( int iPnlID, float fValue,
                            const wxString & rosUnits )
{
  // Check for a valid panel identifier
  if( iPnlID<ID_PNL_FST || iPnlID>ID_PNL_LST )             return( FALSE );

  // Set the panel value
  if( ! roGetPanel( iPnlID ).bSetValue( (float) fValue ) ) return( FALSE );
  m_dfPnlValue[ iPnlID ] = fValue;

  if( ! rosUnits.IsEmpty( ) )
  { // Set the units
    if( ! roGetPanel( iPnlID ).bSetUnits( rosUnits ) )     return( FALSE );
    m_dfPnlValue[ iPnlID ] = roGetPanel( iPnlID ).dfGetValue( );
  }

  return( TRUE );
}

//*****************************************************************************
// Set default values in all of the dialog panels.
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  DlgGenCfg::bSetDefaults( void )
{
  bool  bRtn=TRUE;

  if( ! bSetValue( ID_PNL_AMP,    1.0, wxT("x 1")    ) ) bRtn = FALSE;
  if( ! bSetValue( ID_PNL_OFFSET, 0.0, wxT("V")      ) ) bRtn = FALSE;

  if( ! bSetValue( ID_PNL_FREQ,   0.0, wxT("kHz")    ) ) bRtn = FALSE;
  if( ! bSetValue( ID_PNL_PHASE,  0.0, wxT("Degree") ) ) bRtn = FALSE;

  if( ! bSetValue( ID_PNL_INITV,  0.0, wxT("V")      ) ) bRtn = FALSE;
  if( ! bSetValue( ID_PNL_MINV,   0.0, wxT("V")      ) ) bRtn = FALSE;
  if( ! bSetValue( ID_PNL_MAXV,   0.0, wxT("V")      ) ) bRtn = FALSE;
  if( ! bSetValue( ID_PNL_DELAY,  0.0, wxT("msec")   ) ) bRtn = FALSE;
  if( ! bSetValue( ID_PNL_RISE,   0.0, wxT("msec")   ) ) bRtn = FALSE;
  if( ! bSetValue( ID_PNL_WIDTH,  0.0, wxT("msec")   ) ) bRtn = FALSE;
  if( ! bSetValue( ID_PNL_FALL,   0.0, wxT("msec")   ) ) bRtn = FALSE;
  if( ! bSetValue( ID_PNL_PERIOD, 0.0, wxT("msec")   ) ) bRtn = FALSE;

  return( bRtn );
}

//*****************************************************************************
//
//                             Event Handlers
//
//*****************************************************************************
// Ok button event handler.
//
// Argument List:
//   roEvtCmd - An object holding information about the event (not used)

void  DlgGenCfg::OnBtnOk( wxCommandEvent & roEvtCmd )
{
  if( bValidate( ) )
  {
    GetValues( );
    EndModal( wxID_OK );
    rosGetGenCmd( );
  }
}

//*****************************************************************************
// Cancel button event handler.
//
// Argument List:
//   roEvtCmd - An object holding information about the event (not used)

void  DlgGenCfg::OnBtnCancel( wxCommandEvent & roEvtCmd )
{
  SetValues( );
  EndModal( wxID_CANCEL );
}

//*****************************************************************************
