//*****************************************************************************
//                                 NbkGnuCap.hpp                              *
//                                ---------------                             *
//  Description : Provides a container class for the analysis configuration   *
//                panels for the different analyses provided by GNU-Cap       *
//                electronic circuit simulator.                               *
//  Started     : 05/09/2003                                                  *
//  Last Update : 14/09/2006                                                  *
//  Copyright   : (C) 2003 by MSWaters                                        *
//  Email       : M.Waters@bom.gov.au                                         *
//*****************************************************************************

//*****************************************************************************
//                                                                            *
//    This program is free software; you can redistribute it and/or modify    *
//    it under the terms of the GNU General Public License as published by    *
//    the Free Software Foundation; either version 2 of the License, or       *
//    (at your option) any later version.                                     *
//                                                                            *
//*****************************************************************************

#ifndef NBKGNUCAP_HPP
#define NBKGNUCAP_HPP

// System Includes


// wxWindows Includes

#include <wx/wx.h>

// Application Includes

#include "base/NbkSimrBase.hpp"
#include "gnucap/DlgGnuCapOPT.hpp"
#include "gnucap/PnlGnuCapOP.hpp"
#include "gnucap/PnlGnuCapDC.hpp"
#include "gnucap/PnlGnuCapAC.hpp"
#include "gnucap/PnlGnuCapTR.hpp"
#include "gnucap/PnlGnuCapFO.hpp"

// Local Constant Declarations


//*****************************************************************************

class NbkGnuCap : public NbkSimrBase
{
  private:

    // Analysis pages
    PnlGnuCapOP * m_poPnlGnuCapOP;
    PnlGnuCapDC * m_poPnlGnuCapDC;
    PnlGnuCapAC * m_poPnlGnuCapAC;
    PnlGnuCapTR * m_poPnlGnuCapTR;
    PnlGnuCapFO * m_poPnlGnuCapFO;

    // OPTIONS line setup dialog
    DlgGnuCapOPT  m_oDlgOptions;

  public:

    NbkGnuCap( wxWindow * poParent, wxWindowID oWinID );
    NbkGnuCap( void );
    ~NbkGnuCap( );

    virtual  bool  bClear( void );

    virtual  bool  bLoad  ( Simulation & roSim );
    virtual  bool  bReload( Simulation & roSim );
    virtual  bool  bSave  ( Simulation & roSim );

    virtual  bool  bSetPage( const wxChar * psType );
    virtual  bool  bSetPage( Simulation::eAnaType eType );

    virtual  const  wxString &   rosGetPage( void );
    virtual  Simulation::eAnaType  eGetPage( void );

    virtual  bool  bShowDlgOptions( void );
};

//*****************************************************************************

#endif // NBKGNUCAP_HPP
