//*****************************************************************************
//                              CmdNgSpiceOPT.cpp                             *
//                             -------------------                            *
//  Started     : 23/08/2006                                                  *
//  Last Update : 18/10/2006                                                  *
//  Copyright   : (C) 2006 by M.S.Waters                                      *
//  Email       : M.Waters@bom.gov.au                                         *
//*****************************************************************************

//*****************************************************************************
//                                                                            *
//    This program is free software; you can redistribute it and/or modify    *
//    it under the terms of the GNU General Public License as published by    *
//    the Free Software Foundation; either version 2 of the License, or       *
//    (at your option) any later version.                                     *
//                                                                            *
//*****************************************************************************

#include "ngspice/CmdNgSpiceOPT.hpp"

//*****************************************************************************
// Constructor.

CmdNgSpiceOPT::CmdNgSpiceOPT( void )
{
  bClear( );
}

//*****************************************************************************
// Destructor.

CmdNgSpiceOPT::~CmdNgSpiceOPT( )
{
}

//*****************************************************************************
// Parse the OPTIONS command string.
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  CmdNgSpiceOPT::bParse( void )
{
  wxStringTokenizer  ostk1;
  wxString  os1, os2;
  size_t    szt1;
  long      li1;
  double    df1;

  // Clear the object attributes
  os1 = m_osCmd;
  bClear( );
  m_osCmd = os1;

  // Extract the .OPTIONS line parameter list
  os1 = m_osCmd.Upper( );
  if( (szt1=os1.find( wxT(".OPTIONS") )) == wxString::npos ) return( FALSE );
  os1 = m_osCmd.Right( os1.Length( )-szt1-1 );
  ostk1 = os1;
  if( ostk1.CountTokens( ) <= 0 )                            return( FALSE );

  // Extract each parameter value
  while( ostk1.HasMoreTokens( ) )
  {
    // Extract the field name and the associated value
    os1 = ostk1.GetNextToken( );
    os2 = wxT("");
    if( (szt1 = os1.find( wxT("=") )) != wxString::npos )
    {
      os2 = os1.Right( os1.Length( )-szt1-1 );
      os1 = os1.Left( szt1 );
    }
    ConvertType::bStrToInt( os2, &li1 );
    ConvertType::bStrToFlt( os2, &df1 );

    // Set the display control value
         if( os1 == wxT("ABSTOL")  ) m_fABSTOL  = (float) df1;
    else if( os1 == wxT("BADMOS3") ) m_bBADMOS3 = TRUE;
    else if( os1 == wxT("CHGTOL")  ) m_fCHGTOL  = (float) df1;
    else if( os1 == wxT("DEFAD")   ) m_fDEFAD   = (float) df1;
    else if( os1 == wxT("DEFAS")   ) m_fDEFAS   = (float) df1;
    else if( os1 == wxT("DEFL")    ) m_fDEFL    = (float) df1;
    else if( os1 == wxT("DEFW")    ) m_fDEFW    = (float) df1;
    else if( os1 == wxT("GMIN")    ) m_fGMIN    = (float) df1;
    else if( os1 == wxT("ITL1")    ) m_iITL1    = (int)   li1;
    else if( os1 == wxT("ITL2")    ) m_iITL2    = (int)   li1;
    else if( os1 == wxT("ITL4")    ) m_iITL4    = (int)   li1;
    else if( os1 == wxT("METHOD")  ) m_osMETHOD = os2;
    else if( os1 == wxT("PIVREL")  ) m_fPIVREL  = (float) df1;
    else if( os1 == wxT("PIVTOL")  ) m_fPIVTOL  = (float) df1;
    else if( os1 == wxT("RELTOL")  ) m_fRELTOL  = (float) df1;
    else if( os1 == wxT("TEMP")    ) m_fTEMP    = (float) df1;
    else if( os1 == wxT("TNOM")    ) m_fTNOM    = (float) df1;
    else if( os1 == wxT("TRTOL")   ) m_fTRTOL   = (float) df1;
    else if( os1 == wxT("VNTOL")   ) m_fVNTOL   = (float) df1;
  }

  m_bIsOk = TRUE;

  return( TRUE );
}
       
//*****************************************************************************
// Format the OPTIONS command string.
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  CmdNgSpiceOPT::bFormat( void )
{
  m_osCmd = wxT(".OPTIONS NOPAGE");

  if( m_fABSTOL != NG_ABSTOL )
    m_osCmd << wxT(" ABSTOL=") << wxString::Format( wxT("%.2E"), m_fABSTOL );
  if( m_bBADMOS3 != NG_BADMOS3 )
    m_osCmd << wxT(" BADMOS3");
  if( m_fCHGTOL != NG_CHGTOL )
    m_osCmd << wxT(" CHGTOL=") << wxString::Format( wxT("%.2E"), m_fCHGTOL );
  if( m_fDEFAD != NG_DEFAD )
    m_osCmd << wxT(" DEFAD=")  << wxString::Format( wxT("%.2E"), m_fDEFAD );
  if( m_fDEFAS != NG_DEFAS )
    m_osCmd << wxT(" DEFAS=")  << wxString::Format( wxT("%.2E"), m_fDEFAS );
  if( m_fDEFL != NG_DEFL )
    m_osCmd << wxT(" DEFL=")   << wxString::Format( wxT("%.2E"), m_fDEFL );
  if( m_fDEFW != NG_DEFW )
    m_osCmd << wxT(" DEFW=")   << wxString::Format( wxT("%.2E"), m_fDEFW );
  if( m_fGMIN != NG_GMIN )
    m_osCmd << wxT(" GMIN=")   << wxString::Format( wxT("%.2E"), m_fGMIN );
  if( m_iITL1    != NG_ITL1 )
    m_osCmd << wxT(" ITL1=")   << m_iITL1;
  if( m_iITL2    != NG_ITL2 )
    m_osCmd << wxT(" ITL2=")   << m_iITL2;
  if( m_iITL4    != NG_ITL4 )
    m_osCmd << wxT(" ITL4=")   << m_iITL4;
  if( m_osMETHOD != NG_METHOD )
    m_osCmd << wxT(" METHOD=") << m_osMETHOD;
  if( m_fPIVREL != NG_PIVREL )
    m_osCmd << wxT(" PIVREL=") << wxString::Format( wxT("%.4f"), m_fPIVREL );
  if( m_fPIVTOL != NG_PIVTOL )
    m_osCmd << wxT(" PIVTOL=") << wxString::Format( wxT("%.2E"), m_fPIVTOL );
  if( m_fRELTOL != NG_RELTOL )
    m_osCmd << wxT(" RELTOL=") << wxString::Format( wxT("%.4f"), m_fRELTOL );
  if( m_fTEMP != NG_TEMP )
    m_osCmd << wxT(" TEMP=")   << wxString::Format( wxT("%.2f"), m_fTEMP );
  if( m_fTNOM != NG_TNOM )
    m_osCmd << wxT(" TNOM=")   << wxString::Format( wxT("%.2f"), m_fTNOM );
  if( m_fTRTOL != NG_TRTOL )
    m_osCmd << wxT(" TRTOL=")  << wxString::Format( wxT("%.2E"), m_fTRTOL );
  if( m_fVNTOL != NG_VNTOL )
    m_osCmd << wxT(" VNTOL=")  << wxString::Format( wxT("%.2E"), m_fVNTOL );

  return( TRUE );
}
       
//*****************************************************************************
// Clear the object attributes.
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  CmdNgSpiceOPT::bClear( void )
{
  CmdBase::bClear( );

  m_fABSTOL  = NG_ABSTOL;
  m_fCHGTOL  = NG_CHGTOL;
  m_fDEFAD   = NG_DEFAD;
  m_fDEFAS   = NG_DEFAS;
  m_fDEFL    = NG_DEFL;
  m_fDEFW    = NG_DEFW;
  m_fGMIN    = NG_GMIN;
  m_iITL1    = NG_ITL1;
  m_iITL2    = NG_ITL2;
  m_iITL4    = NG_ITL4;
  m_osMETHOD = NG_METHOD;
  m_fRELTOL  = NG_RELTOL;
  m_fTNOM    = NG_TNOM;
  m_fTRTOL   = NG_TRTOL;
  m_fVNTOL   = NG_VNTOL;

  m_bBADMOS3 = NG_BADMOS3;
  m_fPIVREL  = NG_PIVREL;
  m_fPIVTOL  = NG_PIVTOL;
  m_fTEMP    = NG_TEMP;

  bFormat( ); // Initializes m_osCmd

  return( TRUE );
}

//*****************************************************************************
