//*****************************************************************************
//                                 DlgSrcCfg.cpp                              *
//                                ---------------                             *
//  Started     : 10/05/2005                                                  *
//  Last Update : 24/08/2006                                                  *
//  Copyright   : (C) 2005 by M.S.Waters                                      *
//  Email       : M.Waters@bom.gov.au                                         *
//*****************************************************************************

//*****************************************************************************
//                                                                            *
//    This program is free software; you can redistribute it and/or modify    *
//    it under the terms of the GNU General Public License as published by    *
//    the Free Software Foundation; either version 2 of the License, or       *
//    (at your option) any later version.                                     *
//                                                                            *
//*****************************************************************************

#include "ngspice/DlgSrcCfg.hpp"

//*****************************************************************************
// Implement an event table.

BEGIN_EVENT_TABLE( DlgSrcCfg, wxDialog )

  EVT_BUTTON( ID_BTN_OK,     DlgSrcCfg::OnBtnOk     )
  EVT_BUTTON( ID_BTN_CANCEL, DlgSrcCfg::OnBtnCancel )

END_EVENT_TABLE( )

//*****************************************************************************
// Constructor.
//
// Argument List:
//   poWin - A pointer to the dialog parent window

DlgSrcCfg::DlgSrcCfg( wxWindow * poWin ) : wxDialog( poWin, -1, wxT(""),
    wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE, wxDialogNameStr )
{
  SetTitle( wxT("Voltage Source Setup") );
  SetSize( wxSize( 275, 253 ) );
  Initialize( );
  bClear( );
  bSetDefaults( );
}

//*****************************************************************************
// Destructor.

DlgSrcCfg::~DlgSrcCfg( )
{
}

//*****************************************************************************
// Initialize object attributes.

void  DlgSrcCfg::Initialize( void )
{
  wxPanel * poPnl;

  // Create a panel to contain the pulse parameter controls
  poPnl = new wxPanel( this );

  m_oPnlInitV .bCreate( poPnl, ID_PNL_INITV,  80 );
  m_oPnlPulseV.bCreate( poPnl, ID_PNL_PULSEV, 80 );
  m_oPnlDelay .bCreate( poPnl, ID_PNL_DELAY,  80 );
  m_oPnlRise  .bCreate( poPnl, ID_PNL_RISE,   80 );
  m_oPnlWidth .bCreate( poPnl, ID_PNL_WIDTH,  80 );
  m_oPnlFall  .bCreate( poPnl, ID_PNL_FALL,   80 );
  m_oPnlPeriod.bCreate( poPnl, ID_PNL_PERIOD, 80 );

  m_oPnlInitV .bSetName( wxT("Initial Value") );
  m_oPnlPulseV.bSetName( wxT("Pulsed Value")  );
  m_oPnlDelay .bSetName( wxT("Initial Delay") );
  m_oPnlRise  .bSetName( wxT("Rise Time")     );
  m_oPnlWidth .bSetName( wxT("Pulse Width")   );
  m_oPnlFall  .bSetName( wxT("Fall Time")     );
  m_oPnlPeriod.bSetName( wxT("Period")        );

  m_oPnlInitV .bSetUType( ChoUnits::eUNTS_VOLT  );
  m_oPnlPulseV.bSetUType( ChoUnits::eUNTS_VOLT  );
  m_oPnlDelay .bSetUType( ChoUnits::eUNTS_TIME  );
  m_oPnlRise  .bSetUType( ChoUnits::eUNTS_TIME  );
  m_oPnlWidth .bSetUType( ChoUnits::eUNTS_TIME  );
  m_oPnlFall  .bSetUType( ChoUnits::eUNTS_TIME  );
  m_oPnlPeriod.bSetUType( ChoUnits::eUNTS_TIME  );

  // Create a panel for the buttons
  poPnl = new wxPanel( this );

  m_oBtnOk    .Create( poPnl, ID_BTN_OK,     wxT("OK") );
  m_oBtnCancel.Create( poPnl, ID_BTN_CANCEL, wxT("Cancel") );

  // Layout the of the display objects
  DoLayout( );

  // Initialize temporary storage with the spin control values
  GetValues( );
}

//*****************************************************************************
// Layout the dialog display objects.

void  DlgSrcCfg::DoLayout( void )
{
  wxPanel               * poPnl2, * poPnl3;
  wxBoxSizer  * poSzr1, * poSzr2, * poSzr3;
  wxSizerFlags  oFlags;

  // Create the necessary panels and associated sizers
  poPnl2 = (wxPanel *) m_oPnlInitV.GetParent( );
  poPnl3 = (wxPanel *) m_oBtnOk   .GetParent( );
  poSzr1 = new wxBoxSizer( wxVERTICAL );
  poSzr2 = new wxStaticBoxSizer( wxVERTICAL, poPnl2, wxT("Pulse") );
  poSzr3 = new wxBoxSizer( wxHORIZONTAL );
          SetSizer( poSzr1 );
  poPnl2->SetSizer( poSzr2 );
  poPnl3->SetSizer( poSzr3 );

  // Create and load the panel containing the pulse parameter controls
  oFlags.Center( );
  oFlags.Border( wxTOP, 10 );
  poSzr2->Add( &m_oPnlInitV,  oFlags );
  oFlags.Border( wxLEFT | wxRIGHT, 10 );
  poSzr2->Add( &m_oPnlPulseV, oFlags );
  poSzr2->Add( &m_oPnlDelay,  oFlags );
  poSzr2->Add( &m_oPnlRise,   oFlags );
  poSzr2->Add( &m_oPnlWidth,  oFlags );
  poSzr2->Add( &m_oPnlFall,   oFlags );
  oFlags.Border( wxBOTTOM, 5 );
  poSzr2->Add( &m_oPnlPeriod, oFlags );
  poSzr2->SetSizeHints( poPnl2 );

  // Create and load the panel containing the buttons
  oFlags.Border( wxTOP | wxLEFT | wxRIGHT, 10 );
  poSzr3->Add( &m_oBtnOk,     oFlags );
  poSzr3->AddSpacer( 10 );
  poSzr3->Add( &m_oBtnCancel, oFlags );
  poSzr3->SetSizeHints( poPnl3 );

  // Put it all together
  oFlags.Border( wxALL, 10 );
  poSzr1->Add( poPnl2, oFlags );
  oFlags.Border( wxBOTTOM, 20 );
  poSzr1->Add( poPnl3, oFlags );
  poSzr1->SetSizeHints( this );
}

//*****************************************************************************
// Get a reference to a value panel associated with a control ID number.
//
// Argument List:
//   iPnlID - The control ID number
//
// Return Values:
//   A reference to a value panel

PnlValue & DlgSrcCfg::roGetPanel( int iPnlID )
{
  // No argument tests here so use this function responsibly or suffer the pain
  switch( iPnlID )
  {
    case ID_PNL_INITV  : return( m_oPnlInitV  );
    case ID_PNL_PULSEV : return( m_oPnlPulseV );
    case ID_PNL_DELAY  : return( m_oPnlDelay  );
    case ID_PNL_RISE   : return( m_oPnlRise   );
    case ID_PNL_WIDTH  : return( m_oPnlWidth  );
    case ID_PNL_FALL   : return( m_oPnlFall   );
    case ID_PNL_PERIOD : return( m_oPnlPeriod );
    default            : return( m_oPnlInitV  );
  }
}

//*****************************************************************************
// Set the values in the spin controls from temporary storage.

void  DlgSrcCfg::SetValues( void )
{
  int  i1;

  SetEvtHandlerEnabled( FALSE );

  for( i1=ID_PNL_FST; i1<=ID_PNL_LST; i1++ )
    roGetPanel( i1 ).bSetValue( (float) m_dfPnlValue[ i1 ] );

  SetEvtHandlerEnabled( TRUE );
}

//*****************************************************************************
// Get the values from the spin controls and put them in temporary storage.

void  DlgSrcCfg::GetValues( void )
{
  int  i1;

  SetEvtHandlerEnabled( FALSE );

  for( i1=ID_PNL_FST; i1<=ID_PNL_LST; i1++ )
    m_dfPnlValue[ i1 ] = roGetPanel( i1 ).dfGetValue( );

  SetEvtHandlerEnabled( TRUE );
}

//*****************************************************************************
// Test the values against each other.
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  DlgSrcCfg::bValidate( void )
{
  wxString  osErrMsg;
  double    df1, df2, df3, df4;

  // Test the pulse time values
  df1 = m_oPnlRise  .dfGetValue( );
  df2 = m_oPnlWidth .dfGetValue( );
  df3 = m_oPnlFall  .dfGetValue( );
  df4 = m_oPnlPeriod.dfGetValue( );
  if(      df4 < df1 )
    osErrMsg << wxT("The rise time is greater than the pulse period.\n");
  else if( df4 < df2 )
    osErrMsg << wxT("The pulse width is greater than the pulse period.\n");
  else if( df4 < df3 )
    osErrMsg << wxT("The fall time is greater than the pulse period.\n");
  else if( df4 < df1+df2+df3 )
    osErrMsg << wxT("The pulse period is less then the total pulse width.\n");

  // Test overall logic
  df1 = m_oPnlPulseV.dfGetValue( );
  if( df4>0.0  && df1==0.0 )
    osErrMsg << wxT("Pulse timing has been defined but no level.\n");
  if( df4==0.0 && df1!=0.0 )
    osErrMsg << wxT("Pulse level has been defined but no timing.\n");

  if( ! osErrMsg.IsEmpty( ) )
  {
    wxMessageDialog  oMsgDlg( this, osErrMsg, wxT("Error"), wxOK | wxICON_ERROR );
    oMsgDlg.ShowModal( );
  }

  return( osErrMsg.IsEmpty( ) );
}

//*****************************************************************************
// Parse a independent source definition string.
//
// Argument List:
//   rosCmd - The independent source definition to be parsed
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  DlgSrcCfg::bSetSrcCmd( const wxString & rosCmd )
{
  wxStringTokenizer  ostk1;
  wxString  os1;
  size_t    szt1, szt2;

  // Extract the pulse function parameter list
  os1 = rosCmd.Upper( );
  if( (szt1=os1.find( wxT("PULSE") ))      == wxString::npos ) return( FALSE );
  if( (szt1=rosCmd.find( wxT('('), szt1 )) == wxString::npos ) return( FALSE );
  if( (szt2=rosCmd.find( wxT(')'), szt1 )) == wxString::npos ) return( FALSE );
  os1 = rosCmd.substr( szt1+1, szt2-szt1-1 );
  ostk1 = os1;
  if( ostk1.CountTokens( ) != 7 )                         return( FALSE );

  // Extract each pulse function parameter value
  m_oPnlInitV .bSetValue( ostk1.GetNextToken( ) );
  m_oPnlPulseV.bSetValue( ostk1.GetNextToken( ) );
  m_oPnlDelay .bSetValue( ostk1.GetNextToken( ) );
  m_oPnlRise  .bSetValue( ostk1.GetNextToken( ) );
  m_oPnlFall  .bSetValue( ostk1.GetNextToken( ) );
  m_oPnlWidth .bSetValue( ostk1.GetNextToken( ) );
  m_oPnlPeriod.bSetValue( ostk1.GetNextToken( ) );

  return( TRUE );
}

//*****************************************************************************
// Create the independent voltage source definition string.
//
// Return Values:
//   The generator command

const wxString & DlgSrcCfg::rosGetSrcCmd( void )
{
  m_osSrcCmd = wxT("PULSE( ");

  if( roGetPanel( ID_PNL_INITV  ).IsEnabled( ) )
    m_osSrcCmd << roGetPanel( ID_PNL_INITV  ).rosGetValue( ) << wxT(' ');
  if( roGetPanel( ID_PNL_PULSEV ).IsEnabled( ) )
    m_osSrcCmd << roGetPanel( ID_PNL_PULSEV ).rosGetValue( ) << wxT(' ');
  if( roGetPanel( ID_PNL_DELAY  ).IsEnabled( ) )
    m_osSrcCmd << roGetPanel( ID_PNL_DELAY  ).rosGetValue( ) << wxT(' ');
  if( roGetPanel( ID_PNL_RISE   ).IsEnabled( ) )
    m_osSrcCmd << roGetPanel( ID_PNL_RISE   ).rosGetValue( ) << wxT(' ');
  if( roGetPanel( ID_PNL_FALL   ).IsEnabled( ) )
    m_osSrcCmd << roGetPanel( ID_PNL_FALL   ).rosGetValue( ) << wxT(' ');
  if( roGetPanel( ID_PNL_WIDTH  ).IsEnabled( ) )
    m_osSrcCmd << roGetPanel( ID_PNL_WIDTH  ).rosGetValue( ) << wxT(' ');
  if( roGetPanel( ID_PNL_PERIOD ).IsEnabled( ) )
    m_osSrcCmd << roGetPanel( ID_PNL_PERIOD ).rosGetValue( ) << wxT(' ');

  m_osSrcCmd << wxT(')');

  return( m_osSrcCmd );
}

//*****************************************************************************
// Reset all dialog settings to defaults.
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  DlgSrcCfg::bClear( void )
{
  bool  bRtn=TRUE;

  // Set default spin control values
  if( ! m_oPnlInitV .bClear( ) ) bRtn = FALSE;
  if( ! m_oPnlPulseV.bClear( ) ) bRtn = FALSE;
  if( ! m_oPnlDelay .bClear( ) ) bRtn = FALSE;
  if( ! m_oPnlRise  .bClear( ) ) bRtn = FALSE;
  if( ! m_oPnlWidth .bClear( ) ) bRtn = FALSE;
  if( ! m_oPnlFall  .bClear( ) ) bRtn = FALSE;
  if( ! m_oPnlPeriod.bClear( ) ) bRtn = FALSE;

  return( bRtn );
}

//*****************************************************************************
// Set the value on a panel.
//
// Argument List:
//   iPnlID   - Spin control identifier
//   fValue   - The value to set the spin control
//   rosUnits - The units to display
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  DlgSrcCfg::bSetValue( int iPnlID, float fValue,
                            const wxString & rosUnits )
{
  // Check for a valid panel identifier
  if( iPnlID<ID_PNL_FST || iPnlID>ID_PNL_LST )             return( FALSE );

  // Set the panel value
  if( ! roGetPanel( iPnlID ).bSetValue( (float) fValue ) ) return( FALSE );
  m_dfPnlValue[ iPnlID ] = fValue;

  if( ! rosUnits.IsEmpty( ) )
  { // Set the units
    if( ! roGetPanel( iPnlID ).bSetUnits( rosUnits ) )     return( FALSE );
    m_dfPnlValue[ iPnlID ] = roGetPanel( iPnlID ).dfGetValue( );
  }

  return( TRUE );
}
//*****************************************************************************
// Enable or disable a value panel.
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  DlgSrcCfg::bEnablePanel( int iPnlID, bool bState )
{
  if( iPnlID<ID_PNL_FST || iPnlID>ID_PNL_LST ) return( FALSE );

  if( roGetPanel( iPnlID ).Enable( bState ) )  return( FALSE );

  return( TRUE );
}

//*****************************************************************************
// Set default values in all of the dialog panels.
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  DlgSrcCfg::bSetDefaults( void )
{
  bool  bRtn=TRUE;

  if( ! bSetValue( ID_PNL_INITV,  0.0, wxT("V")    ) ) bRtn = FALSE;
  if( ! bSetValue( ID_PNL_PULSEV, 0.0, wxT("V")    ) ) bRtn = FALSE;
  if( ! bSetValue( ID_PNL_DELAY,  0.0, wxT("msec") ) ) bRtn = FALSE;
  if( ! bSetValue( ID_PNL_RISE,   0.0, wxT("msec") ) ) bRtn = FALSE;
  if( ! bSetValue( ID_PNL_WIDTH,  0.0, wxT("msec") ) ) bRtn = FALSE;
  if( ! bSetValue( ID_PNL_FALL,   0.0, wxT("msec") ) ) bRtn = FALSE;
  if( ! bSetValue( ID_PNL_PERIOD, 0.0, wxT("msec") ) ) bRtn = FALSE;

  return( bRtn );
}

//*****************************************************************************
//
//                             Event Handlers
//
//*****************************************************************************
// Ok button event handler.
//
// Argument List:
//   roEvtCmd - An object holding information about the event (not used)

void  DlgSrcCfg::OnBtnOk( wxCommandEvent & roEvtCmd )
{
  if( bValidate( ) )
  {
    GetValues( );
    EndModal( wxID_OK );
    rosGetSrcCmd( );
  }
}

//*****************************************************************************
// Cancel button event handler.
//
// Argument List:
//   roEvtCmd - An object holding information about the event (not used)

void  DlgSrcCfg::OnBtnCancel( wxCommandEvent & roEvtCmd )
{
  SetValues( );
  EndModal( wxID_CANCEL );
}

//*****************************************************************************
